package contacter;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import com.google.gdata.data.BaseEntry;
import com.google.gdata.data.contacts.ContactEntry;
import com.google.gdata.data.contacts.ContactGroupEntry;
import com.google.gdata.data.extensions.Name;

public class ContactLink {
    private static final Logger log = Logger.getLogger(ContactLink.class.getName());
	private ContactLinksTable tableRecord;
	private PersistenceManager pm;
	private String idOnRepo, idOnUser, name, userIdTo, userIdFrom;
	private Query query;
	private boolean isContact;
	private ContactEntry contactEntry;
	private ContactGroupEntry contactGroupEntry;
	
	@SuppressWarnings("unchecked")
	public ContactLink(BaseEntry<?> baseEntry, String userIdTo){
		if (baseEntry.getId().contains("/feeds/contacts/")){
			this.isContact = true;
			contactEntry = new ContactEntry(baseEntry);
		} else
		{
			contactGroupEntry = new ContactGroupEntry(baseEntry);
		}
			
		
		// extract email address + name
		if (isContact){
			this.userIdFrom = contactEntry.getId().replace("http://www.google.com/m8/feeds/contacts/", "");
			this.idOnRepo = contactEntry.getId();
		    if (contactEntry.hasName()) {
		        Name name = contactEntry.getName();
		        if (name.hasFullName()) {
		        	this.name = name.getFullName().getValue();
		        }
		    }
		} else
		{
			this.userIdFrom = contactGroupEntry.getId().replace("http://www.google.com/m8/feeds/groups/", "");
			this.idOnRepo = contactGroupEntry.getId();
        	this.name = contactGroupEntry.getPlainTextContent();
		}
		
		this.userIdFrom = this.userIdFrom.replace(this.userIdFrom.substring(this.userIdFrom.indexOf("/"),this.userIdFrom.length()),""); 
		// replace %40 with "@"
		StringBuffer userIdFromSB = new StringBuffer(this.userIdFrom);
		userIdFromSB = userIdFromSB.replace(this.userIdFrom.indexOf("%40"),this.userIdFrom.indexOf("%40") + 3, "@");
		this.userIdFrom = userIdFromSB.toString();

	    
		//String idOnRepoParam = idOnRepo;
		this.userIdTo = userIdTo;
		this.pm = PMF.get().getPersistenceManager();
		query = pm.newQuery(ContactLinksTable.class);
		query.setFilter("idOnRepo == idOnRepoParam");
		query.declareParameters("String idOnRepoParam"); 
		
	    try {
			List<ContactLinksTable> results = (List<ContactLinksTable>) query.execute(idOnRepo);
			switch (results.size()){
				case 0: 	
					this.tableRecord = null;
					this.idOnUser = null;
					break;
				case 1:
					this.tableRecord = results.get(0);
					if (userIdTo.equals("test3@gmail.com")){this.idOnUser = tableRecord.getTest3_gmail();}
					if (userIdTo.equals("test4@gmail.com")){this.idOnUser = tableRecord.getTest4_gmail();}
					break;
				default: 
					log.log(Level.SEVERE, "Problem in getDestId() " + results.size() + " records found for " + idOnRepo);
					this.tableRecord = null;
					this.idOnUser = null;
			}
	    }finally{query.closeAll();}
	}


	public void setIdOnUser(String idOnUser){
		this.idOnUser = idOnUser;
	}
	public String getIdOnUser(){
		return this.idOnUser;
	}
	
	public void setName(String name){
		this.name = name;
	}
	public String getName(){
		return this.name;
	}

	public void setUserIdTo(String userIdTo){
		this.userIdTo = userIdTo;
	}
	public String getUserIdTo(){
		return this.userIdTo;
	}
	public boolean getIsContact(){
		return this.isContact;
	}
	public String getIdOnRepo(){
		return this.idOnRepo;
	}

	public void commit(String idOnUser){ 
		this.idOnUser = idOnUser;
	    try {
	    	if (tableRecord == null){
					if (userIdTo.equals("test3@gmail.com")){tableRecord = new ContactLinksTable(userIdFrom,idOnRepo,idOnUser,null,null,null,null,null,null,null,null,null,name);}
					if (userIdTo.equals("test4@gmail.com")){tableRecord = new ContactLinksTable(userIdFrom,idOnRepo,null,idOnUser,null,null,null,null,null,null,null,null,name);}
	    	} else{
					if (userIdTo.equals("test3@gmail.com")){tableRecord.setTest3_gmail(idOnUser);}
					if (userIdTo.equals("test4@gmail.com")){tableRecord.setTest4_gmail(idOnUser);}
	    	}

	    	// If all Ids are blank, delete the record 
	    	if (tableRecord.getTest3_gmail() == null && tableRecord.getTest4_gmail() == null && tableRecord.getIdOnUser3() == null &&
	    			tableRecord.getIdOnUser4() == null && tableRecord.getIdOnUser5() == null && tableRecord.getIdOnUser6() == null
	    			 && tableRecord.getIdOnUser7() == null && tableRecord.getIdOnUser8() == null && tableRecord.getIdOnUser9() == null
	    			 && tableRecord.getIdOnUser10() == null){
	    		pm.deletePersistent(tableRecord);
	    	}
	    } finally {
			pm.makePersistent(tableRecord);
			pm.close();
		}
	}
}
