package contacter;

import java.io.IOException;
import java.net.URL;
import java.util.Date;
import java.util.logging.Logger;

import com.google.gdata.client.Query;
import com.google.gdata.client.contacts.ContactsService;
import com.google.gdata.data.DateTime;
import com.google.gdata.data.contacts.ContactEntry;
import com.google.gdata.data.contacts.ContactFeed;
import com.google.gdata.data.contacts.ContactGroupEntry;
import com.google.gdata.data.contacts.ContactGroupFeed;
import com.google.gdata.data.contacts.GroupMembershipInfo;
import com.google.gdata.data.extensions.Name;
import com.google.gdata.util.ServiceException;

public class ContactsActions {
  private static final Logger log = Logger.getLogger(ContactsActions.class.getName());
  private static String[] saveExistingGroupHref;
//  protected static ContactGroupEntry[] contactGroupArray = new ContactGroupEntry[50];
	

  public static ContactFeed getRecentContactUpdates(
		    ContactsService myService, Date startTime, URL feedUrl, boolean showDeleted) throws ServiceException,
		    IOException {
		  // Create query and submit a request
	  	  Query myQuery = new Query(feedUrl);
		  myQuery.setUpdatedMin(new DateTime(startTime));
		  myQuery.setMaxResults(500);
		  if (showDeleted)
			  myQuery.setStringCustomParameter("showdeleted","true"); 
		  return myService.query(myQuery, ContactFeed.class);
		}
  public static ContactGroupFeed getRecentGroupUpdates(
		    ContactsService myService, Date startTime, URL feedUrl, boolean showDeleted) throws ServiceException,
		    IOException {
		  // Create query and submit a request
	  	  Query myQuery = new Query(feedUrl);
		  myQuery.setUpdatedMin(new DateTime(startTime));
		  //myQuery.setMaxResults(1000);
		  if (showDeleted)
			  myQuery.setStringCustomParameter("showdeleted","true"); 
		  return myService.query(myQuery, ContactGroupFeed.class);
		}
  
  public static ContactFeed getAllContacts(
		    ContactsService myService, URL feedUrl) throws ServiceException,
		    IOException {
		  // Create query and submit a request
	  	  Query myQuery = new Query(feedUrl);
		  myQuery.setMaxResults(1000);
		  return myService.query(myQuery, ContactFeed.class);
		}
  
public static void copyAllContacts(ContactsService serviceFrom, String userIdFrom, ContactsService serviceTo
		, String userIdTo) throws ServiceException, IOException {
  
	  // Request the feed
	  URL fromUrl = new URL("http://www.google.com/m8/feeds/contacts/default/full?max-results=1000");
	  URL toUrl = new URL("http://www.google.com/m8/feeds/contacts/" + userIdTo + "/full?max-results=1000");
	  ContactFeed fromResultFeed = serviceFrom.getFeed(fromUrl, ContactFeed.class);
	  //ContactFeed toResultFeed = toService.getFeed(feedUrl, ContactFeed.class);
	  log.info("Attempting to copy " + fromResultFeed.getEntries().size() 
			  + " entries from " + userIdFrom + " into " + userIdTo);
	  
      int nbUpdt = 0;

      //for (int i = 0; i < fromResultFeed.getEntries().size(); i++) {
      for (int i = 0; i < 50; i++) {
    	  //storeNewEntry(fromResultFeed.getEntries().get(i),userIdFrom, userIdTo, serviceTo, toUrl );
    	  nbUpdt++;
	  }
      log.info("Copied " + nbUpdt + " entries into " + userIdTo);
}


public static String[] fixGroups(ContactsService serviceFrom, ContactEntry entry, 
		String userIdFrom, String userIdTo) throws ServiceException, IOException{
	int sizeBefore = entry.getGroupMembershipInfos().size();
	ContactLink groupLink;
	saveExistingGroupHref = new String[sizeBefore];
	StringBuilder groupHref;
		
	
	// Go over all of the contact's groups 
    for (int idx=0;idx < sizeBefore;idx++){ 
    	GroupMembershipInfo group = entry.getGroupMembershipInfos().get(idx);
    	saveExistingGroupHref[idx] = group.getHref(); // Save existing contact links
       
    	// Find appropriate group in the array
    	groupLink = null;
    	for (int i = 0; i < DoSyncServlet.contactGroupArray.length; i++) {
	    	if (DoSyncServlet.contactGroupArray[i].getId().equals(group.getHref())){

	    		if (DoSyncServlet.contactGroupArray[i].hasSystemGroup()) // | (group.hasDeleted()))
	    		{
	    			groupHref = new StringBuilder(group.getHref());
	    	        // Manipulate the data
	    	        groupHref = groupHref.replace(groupHref.indexOf("%40"),groupHref.indexOf("%40") + 3, "@");
	    	        groupHref = groupHref.replace(groupHref.indexOf(userIdFrom),groupHref.indexOf(userIdFrom) + userIdFrom.length(), userIdTo);
	    	        groupHref = groupHref.replace(groupHref.indexOf("@"),groupHref.indexOf("@") + 1, "%40");
	    	        group.setHref(groupHref.toString());
	    	    }
	    		else{
	    			groupLink = new ContactLink(DoSyncServlet.contactGroupArray[i], userIdTo);
	    			group.setHref(groupLink.getIdOnUser());
	    		}
	    		break;
	    	}
    	}
    }
    return saveExistingGroupHref;
}

static void storeNewEntry(ContactsService serviceFrom,ContactEntry CRUDentry, 
		String userIdFrom, ContactsService serviceTo, URL feedUrlTo, ContactLink contactLink) 
	throws IOException,ServiceException{
	ContactEntry entryToAdd = new ContactEntry(CRUDentry);
	
	if (entryToAdd.hasGroupMembershipInfos()) {
		saveExistingGroupHref = ContactsActions.fixGroups(serviceFrom, CRUDentry,userIdFrom,contactLink.getUserIdTo());
	}
	
	entryToAdd = serviceTo.insert(feedUrlTo, entryToAdd); 
		
	if (entryToAdd.hasGroupMembershipInfos()) {
	    for (int idx=0;idx < CRUDentry.getGroupMembershipInfos().size();idx++){ // Put contact links back
	    	GroupMembershipInfo group = CRUDentry.getGroupMembershipInfos().get(idx);
	    	group.setHref(saveExistingGroupHref[idx]);
	    }
	}
	// -- Save destination key on source record
	contactLink.commit(entryToAdd.getId());
}

		
static void storeNewGroupEntry(ContactGroupEntry CRUDGroupEntry, String userIdFrom, ContactsService serviceTo, 
		URL feedUrlTo, ContactLink contactLink) throws IOException,ServiceException{
	ContactGroupEntry entryToAdd = new ContactGroupEntry(CRUDGroupEntry);
	
	entryToAdd = serviceTo.insert(feedUrlTo, entryToAdd); 

	// -- Save destination key on source record
	contactLink.commit(entryToAdd.getId());
}


  public static void printAllContacts(ContactsService myService)
    throws ServiceException, IOException {
	  
  // Request the feed
  URL feedUrl = new URL("http://www.google.com/m8/feeds/contacts/default/full?max-results=1000");
  ContactFeed resultFeed = myService.getFeed(feedUrl, ContactFeed.class);
  
  // Print the results
  log.info("*** PRINTING CONTACTS FOR " + resultFeed.getTitle().getPlainText() + " ***");
  for (int i = 0; i < resultFeed.getEntries().size(); i++) {
    ContactEntry entry = resultFeed.getEntries().get(i);

    if (entry.hasName()) {
      Name name = entry.getName();
      if (name.hasFullName()) {
        String fullNameToDisplay = name.getFullName().getValue();
        if (name.getFullName().hasYomi()) {
          fullNameToDisplay += " (" + name.getFullName().getYomi() + ")";
        }
        System.out.println("-------------------------------------------------------------------------");
        System.out.println(fullNameToDisplay + " - ID = " + entry.getId());
      } /*else {
        System.out.println("\t\t (no full name found)");
      }
      if (name.hasNamePrefix()) {
        System.out.println("\t\t" + name.getNamePrefix().getValue());
      } else {
        System.out.println("\t\t (no name prefix found)");
      }
      if (name.hasGivenName()) {
        String givenNameToDisplay = name.getGivenName().getValue();
        if (name.getGivenName().hasYomi()) {
          givenNameToDisplay += " (" + name.getGivenName().getYomi() + ")";
        }
        System.out.println("\t\t" + givenNameToDisplay);
      } else {
        System.out.println("\t\t (no given name found)");
      }
      if (name.hasAdditionalName()) {
        String additionalNameToDisplay = name.getAdditionalName().getValue();
        if (name.getAdditionalName().hasYomi()) {
          additionalNameToDisplay += " (" + name.getAdditionalName().getYomi() + ")";
        }
        System.out.println("\t\t" + additionalNameToDisplay);
      } else {
        System.out.println("\t\t (no additional name found)");
      }
      if (name.hasFamilyName()) {
        String familyNameToDisplay = name.getFamilyName().getValue();
        if (name.getFamilyName().hasYomi()) {
          familyNameToDisplay += " (" + name.getFamilyName().getYomi() + ")";
        }
        System.out.println("\t\t" + familyNameToDisplay);
      }/* else {
        System.out.println("\t\t (no family name found)");
      }
      if (name.hasNameSuffix()) {
        System.out.println("\t\t" + name.getNameSuffix().getValue());
      } /*else {
        System.out.println("\t\t (no name suffix found)");
      }*/
    } else {
      System.out.println("\t (no name found)");
    }

   /* if (entry.hasOccupation())
    	System.out.print(" OCCUPATION= " + entry.getOccupation().getValue());
    //System.out.println("Email addresses:");
    for (Email email : entry.getEmailAddresses()) {
      System.out.print(" " + email.getAddress());
      if (email.getRel() != null) {
        System.out.print(" rel:" + email.getRel());
      }
      if (email.getLabel() != null) {
        System.out.print(" label:" + email.getLabel());
      }
      if (email.getPrimary()) {
        System.out.print(" (primary) ");
      }
      System.out.print("\n");
    }*/

    /*System.out.println("IM addresses:");
    for (Im im : entry.getImAddresses()) {
      System.out.print(" " + im.getAddress());
      if (im.getLabel() != null) {
        System.out.print(" label:" + im.getLabel());
      }
      if (im.getRel() != null) {
        System.out.print(" rel:" + im.getRel());
      }
      if (im.getProtocol() != null) {
        System.out.print(" protocol:" + im.getProtocol());
      }
      if (im.getPrimary()) {
        System.out.print(" (primary) ");
      }
      System.out.print("\n");
    }*/

    System.out.println("Groups:");
    for (GroupMembershipInfo group : entry.getGroupMembershipInfos()) {      
      String groupHref = group.getHref();
      System.out.println("  Id: " + groupHref);
    }

    /*for (ExtendedProperty property : entry.getExtendedProperties()) {
      if (property.getValue() != null) {
        System.out.println("Extended Properties (name) = " + property.getName() + ". (value) = " + 
            property.getValue());
      } /*else if (property.getXmlBlob() != null) {
        System.out.println("  " + property.getName() + "(xmlBlob)= " + 
            property.getXmlBlob().getBlob());
      }
    }*/

  }
  // GROUPS!!!
  // Request the feed
  log.info("*** PRINTING GROUPS FOR " + resultFeed.getTitle().getPlainText() + " ***");
  feedUrl = new URL("http://www.google.com/m8/feeds/groups/default/full?max-results=1000");
  ContactGroupFeed resultGroupFeed = myService.getFeed(feedUrl, ContactGroupFeed.class);
  // Print the results

  for (int i = 0; i < resultGroupFeed.getEntries().size(); i++) {
    ContactGroupEntry groupEntry = resultGroupFeed.getEntries().get(i);
    System.out.println(groupEntry.getPlainTextContent() + " - id: " + groupEntry.getId() + " SYS? " + groupEntry.hasSystemGroup());
  }
}


}
