package contacter;

import java.io.IOException;
import java.net.URL;
import java.util.logging.Logger;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.google.appengine.api.datastore.DatastoreTimeoutException;
import com.google.gdata.client.contacts.ContactsService;
import com.google.gdata.data.DateTime;
import com.google.gdata.data.PlainTextConstruct;
import com.google.gdata.data.contacts.ContactEntry;
import com.google.gdata.data.contacts.ContactFeed;
import com.google.gdata.data.contacts.ContactGroupEntry;
import com.google.gdata.data.contacts.ContactGroupFeed;
import com.google.gdata.util.ServiceException;

public class DoSyncServlet extends HttpServlet {
    private static final Logger log = Logger.getLogger(DoSyncServlet.class.getName());
    private static String userIdFrom, userIdTo;
    private boolean batch;
    private static boolean showDeleted;
    private static ContactsService serviceTo = null, serviceFrom = null;
    private static URL feedUrlTo, feedUrlFrom;
    private final static int BASE_NBR = 20;
    protected static ContactGroupEntry[] contactGroupArray = new ContactGroupEntry[50];
    private static int i, nbrUpdt, passNbr, nbrGrpUpdt;
    private static LastProcess lastProcess;
    private static ContactFeed destFeed, updatesFromFeed;
    
    public void doGet(HttpServletRequest req, HttpServletResponse resp)
	throws IOException {
    	batch = true;
    	doPost(req,resp);
    }
    @SuppressWarnings("static-access")
	public void doPost(HttpServletRequest req, HttpServletResponse resp)
    throws IOException {
    	try
    	{
	        boolean showDeleted = false;
	        nbrGrpUpdt=0;
		    userIdFrom = 
			   req.getParameter("userIdFrom").substring(0, req.getParameter("userIdFrom").indexOf("(")).trim();
		    userIdTo = 
			   req.getParameter("userIdTo").substring(0, req.getParameter("userIdTo").indexOf("(")).trim();
		    if (req.getParameter("passNbr") == null)
		    {passNbr=0; nbrUpdt=0;}
		    else {passNbr = Integer.parseInt(req.getParameter("passNbr"));
		    	nbrUpdt= passNbr * BASE_NBR;};
		    if (req.getParameter("showDeleted") != null && req.getParameter("showDeleted").equals("on")){
		    	showDeleted = true;
		    }
	        
		   // -- Get services and authorization for both FROM and TO users
		   serviceTo = new OfflineAuthentication().getService(userIdTo);
		   serviceFrom = new OfflineAuthentication().getService(userIdFrom);
		   
		   // -- Get "last process" record
		   lastProcess = new LastProcess(userIdFrom,userIdTo);
		   
		   
		   if (passNbr==0)
		   {
			   // ****************************************************************
			   // -- Browse the feed and perform updates/deletes/stores for GROUPS
			   // ****************************************************************
			   // -- Request the feeds
			   feedUrlFrom = new URL("http://www.google.com/m8/feeds/groups/" + userIdFrom + "/full");

			   
			   ContactGroupFeed updatesFromGroupFeed = serviceFrom.getFeed(feedUrlFrom,ContactGroupFeed.class);
			   for (i = 0; i < updatesFromGroupFeed.getEntries().size(); i++) { 
				    contactGroupArray[i] = updatesFromGroupFeed.getEntries().get(i);
			   }
			   
		   
			   //ContactGroupFeed updatesFromGroupFeed = null;
			   updatesFromGroupFeed = ContactsActions.getRecentGroupUpdates
				   		(serviceFrom,lastProcess.getLastProcessedDate(),feedUrlFrom, showDeleted);
			   ContactGroupFeed destGroupFeed = null;
			   if (updatesFromGroupFeed.getEntries().size() > 0)
			   {
			   	   feedUrlTo = new URL("http://www.google.com/m8/feeds/groups/" + userIdTo + "/full");
				   destGroupFeed=serviceTo.getFeed(feedUrlTo, ContactGroupFeed.class);
			   }
			   
			   int i;
			   for (i = 0; i < updatesFromGroupFeed.getEntries().size(); i++) {
				   ContactLink contactLink = null;
				   ContactGroupEntry CRUDGroupEntry = updatesFromGroupFeed.getEntries().get(i);
				   
				   contactLink = new ContactLink(CRUDGroupEntry, userIdTo);
				   // Exclude SYSTEM groups
				   if (CRUDGroupEntry.hasSystemGroup()){
					   continue;
				   }
				   
				   // CRUD entry HAS a foreign key and is NOT marked as DELETED
				   // --> Update it from DESTINATION ***************************************************
				   if (contactLink.getIdOnUser() != null && !CRUDGroupEntry.hasDeleted()){
					   log.info("GROUP - Syncing " + contactLink.getName() + " (UPDATE) - #" + i);
					   for (int idx = 0; idx < destGroupFeed.getEntries().size(); idx++) {
						   ContactGroupEntry groupEntryToUpdate = destGroupFeed.getEntries().get(idx);
						 if (groupEntryToUpdate.getId().equals(contactLink.getIdOnUser())){
							 
							groupEntryToUpdate.setTitle(new PlainTextConstruct(CRUDGroupEntry.getPlainTextContent()));
							contactLink.commit(groupEntryToUpdate.getId());
							serviceTo.update(new URL(groupEntryToUpdate.getEditLink().getHref()), groupEntryToUpdate);
							break; 
						 }
					   }
				   }
				   
				   // CRUD entry HAS a foreign key and is marked as DELETED
				   // --> Delete it from DESTINATION ***************************************************
				   if (contactLink.getIdOnUser() != null && CRUDGroupEntry.hasDeleted()){
					   log.info("GROUP - Syncing " + contactLink.getName() + " (DELETE) - #" + i);
					   for (int idx = 0; idx < destGroupFeed.getEntries().size(); idx++) {
					     ContactGroupEntry groupEntryToDelete = destGroupFeed.getEntries().get(idx);
						 if (groupEntryToDelete.getId().equals(contactLink.getIdOnUser())){
							 groupEntryToDelete.delete();
							 contactLink.commit(null);
							 break;
						 } 
					   }
				   }
			   
				   // CRUD entry DOES NOT have a foreign key and is NOT marked as DELETED
				   // --> Store it on DESTINATION ***************************************************
				   if (contactLink.getIdOnUser() == null && !CRUDGroupEntry.hasDeleted() && !CRUDGroupEntry.hasSystemGroup()){
					   log.info("GROUP - Syncing " + contactLink.getName() + " (STORE) - #" + i);
					   ContactsActions.storeNewGroupEntry(CRUDGroupEntry, userIdFrom, serviceTo, feedUrlTo, contactLink);
				   }
				   nbrGrpUpdt++;
			   }
		   }

		   
		   // ******************************************************************
		   // -- Browse the feed and perform updates/deletes/stores for CONTACTS
		   // ******************************************************************
		   
		   // -- Request the feed URLs
		   feedUrlFrom = new URL("http://www.google.com/m8/feeds/contacts/" + userIdFrom + "/full");

		   
		   // -- Get the feed that has the updates since the last process ran
		   if (showDeleted)
		   {
			   processContacts(passNbr, true);
			   processContacts(passNbr, false);
		   }
		   else
		   {
			   processContacts(passNbr, false);
		   }
		   
		   
 
		   if (i >= BASE_NBR * (passNbr + 1))
		   {
			   log.info("...Redirecting to recycle.jsp... i= " + i);
			   resp.sendRedirect("/recycle.jsp?userIdFrom=" + userIdFrom + "&userIdTo=" + userIdTo + "&passNbr=" + passNbr); 
		   }
		   else
		   {
			   // Update the last update date
			   long dateTemp = new DateTime().now().getValue();
			   dateTemp -= 65000; // go back 1 minute 5 seconds
			   DateTime lastUpdated =  new DateTime(dateTemp);
			   //DateProcessing.updateLastProcessDate(lastProcessTable, lastUpdated);
			   lastProcess.commit(lastUpdated,nbrUpdt);
			   
			   if (nbrUpdt > 0){
				   log.info("Sync " + userIdFrom + " --> " + userIdTo + " (in:" + lastProcess.getLastProcessedDate() + ")" + " (out:" + lastUpdated + ") ** Number of Updates: " + nbrUpdt);
			   }
			   if (!batch)
					resp.sendRedirect("/success.jsp?from=" + req.getParameter("userIdFrom") + "&to=" + 
							  req.getParameter("userIdTo") + "&nb=" + nbrUpdt + "&nbG=" + nbrGrpUpdt);
		   }
    	}
   	 	catch (DatastoreTimeoutException dte){dte.printStackTrace();}//throw new RuntimeException(dte);}  
   	 	catch (Exception e){
 		   ErrorHandling.sendEMail(ErrorHandling.stackToString(e));
		   throw new RuntimeException(e); 
   	 	}
    }
    
    private static int processContacts(int passNbr, boolean showDeleted) throws ServiceException, IOException
    {
 	   updatesFromFeed = ContactsActions.getRecentContactUpdates
  		(serviceFrom,lastProcess.getLastProcessedDate(),feedUrlFrom, showDeleted);
 	   
 	   if (updatesFromFeed.getEntries().size() > 0)
 	   {
		   feedUrlTo = new URL("http://www.google.com/m8/feeds/contacts/" + userIdTo + "/full?max-results=500");
		   destFeed = serviceTo.getFeed(feedUrlTo, ContactFeed.class);
 	   }
	   
	   for (i = 0; i < updatesFromFeed.getEntries().size(); i++) {
		   if ( i >= (BASE_NBR * passNbr) && i < BASE_NBR * (passNbr + 1)) 
		   {
			   ContactLink contactLink = null;
			   ContactEntry CRUDentry = updatesFromFeed.getEntries().get(i);
			   
			   contactLink = new ContactLink(CRUDentry, userIdTo);
			          
			   // CRUD entry HAS a foreign key and is NOT marked as DELETED
			   // --> Update it from DESTINATION ***************************************************
			   //if (contactLink.getIdOnUser() != null && !CRUDentry.hasDeleted()){
			   if (contactLink.getIdOnUser() != null && !showDeleted && !CRUDentry.hasDeleted()){
				   log.info("CONTACT - Syncing " + contactLink.getName() + " (UPDATE) - #" + i);
				   for (int idx = 0; idx < destFeed.getEntries().size(); idx++) {
				     ContactEntry entryToUpdate = destFeed.getEntries().get(idx);
					 if (entryToUpdate.getId().equals(contactLink.getIdOnUser())){
					 	entryToUpdate.delete();
					 	ContactsActions.storeNewEntry
					 	(serviceFrom, CRUDentry, userIdFrom, serviceTo, feedUrlTo, contactLink);
						break; 
					 }
				   }
			   }
			   
			   // CRUD entry HAS a foreign key and is marked as DELETED
			   // --> Delete it from DESTINATION ***************************************************
			   //if (contactLink.getIdOnUser() != null && CRUDentry.hasDeleted()){
			   if (contactLink.getIdOnUser() != null && showDeleted && CRUDentry.hasDeleted()){
				   log.info("CONTACT - Syncing " + contactLink.getName() + " (DELETE) - #" + i);
				   for (int idx = 0; idx < destFeed.getEntries().size(); idx++) {
				     ContactEntry entryToDelete = destFeed.getEntries().get(idx);
					 if (entryToDelete.getId().equals(contactLink.getIdOnUser())){
						 entryToDelete.delete();
						 contactLink.commit(null);
						 break;
					 } 
				   }
			   }
		   
			   // CRUD entry DOES NOT have a foreign key and is NOT marked as DELETED
			   // --> Store it on DESTINATION ***************************************************
			   //if (contactLink.getIdOnUser() == null && !CRUDentry.hasDeleted()){
			   if (contactLink.getIdOnUser() == null && !showDeleted && !CRUDentry.hasDeleted()){
				   log.info("CONTACT - Syncing " + contactLink.getName() + " (STORE) - #" + i);
				   ContactsActions.storeNewEntry(serviceFrom, CRUDentry, userIdFrom, serviceTo, feedUrlTo, contactLink);
			   }
			   
			   nbrUpdt++;
		   } 
		   else
		   {	
			   if (i >= BASE_NBR * (passNbr + 1)){break;}
		   }
	   }
	   return nbrUpdt;
    }
}
